#pragma once

#include "Betriebssystem.h"

struct ID3D11Buffer;
enum D3D11_BIND_FLAG;

namespace Framework
{
    class Render3D;

    // Eine Schnittstelle zwischen dem Arbeitsspeicher und dem Grafikspeicher
    class DXBuffer
    {
    private:
        ID3D11Buffer *buffer;
        D3D11_BIND_FLAG bf;
        void *data;
        bool gendert;
        int ln;
        int altLn;
        int elLn;

    protected:
        int ref;

    public:
        // Konstruktor
        //  bind: Der verwendungszweck des Buffers. Beispiel: D3D11_BIND_INDEX_BUFFER, D3D11_BIND_VERTEX_BUFFER.
        //  eLn: Lnge eines einzelnen Elements in Bytes
        __declspec( dllexport ) DXBuffer( D3D11_BIND_FLAG bind, int eLn );
        // Destruktor
        __declspec( dllexport ) ~DXBuffer();
        // Setzt den gendert flg, so das beim nchsten auruf von 'kopieren' die daten neu kopiert werden
        __declspec( dllexport ) void setGendert();
        // ndert die lnge des Buffers beim nchsten aufruf von 'kopieren'
        //  ln: Die Lnge in Bytes
        __declspec( dllexport ) void setLnge( int ln );
        // Legt fest, was beim nchsten aufruf von 'kopieren' kopiert wird
        //  data: Ein zeiger auf die Daten
        __declspec( dllexport ) void setData( void *data );
        // Kopiert die Daten in den Buffer, fals sie sich verndert haben
        //  zRObj: Das Objekt, mit dem die Grafikkarte angesprochen wird
        __declspec( dllexport ) void copieren( Render3D *zRObj );
        // Gibt die Lnge eines Elementes in bytes zurck
        __declspec( dllexport ) int getElementLnge() const;
        // Gibt den Buffer zurck
        __declspec( dllexport ) ID3D11Buffer *zBuffer() const;
        // Gibt die Anzahl der Elemente im Buffer zurck
        __declspec( dllexport ) int getElementAnzahl() const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) DXBuffer *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) virtual DXBuffer *release();
    };

    // Ein Buffer von Eckpunkten eines 3D Models
    class DXVertexBuffer : public DXBuffer
    {
    public:
        // Konstruktor
        // eSize: Die Lnge eines Elementes in Bytes
        __declspec( dllexport ) DXVertexBuffer( int eSize );
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) virtual DXBuffer *release();
    };

    // Ein Buffer von Indizes aus dem Buffer mit Eckpunkten, wovon immer drei ein Dreieck ergeben, das gezeichnet wird
    class DXIndexBuffer : public DXBuffer
    {
    public:
        // Konstruktor
        // eSize: Die Lnge eines Elementes in Bytes
        __declspec( dllexport ) DXIndexBuffer( int eSize );
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) virtual DXBuffer *release();
    };
}