#ifndef Globals_H
#define Globals_H

#include "Punkt.h"

#ifndef Global
#define Global extern
#endif

namespace Framework
{
    class ThreadRegister; // Thread.h
    class Thread; // Thread.h
#ifdef WIN32
    class Maus; // Maus.h
    class WFensterArray; // Fenster.h
    class Model3DList; // Model3DList.h
    class TexturList; // TexturList.h

    Global WFensterArray WFensterA;
    Global bool MausTrack;
    Global bool MausStand[ 3 ];
    Global bool TastenStand[ 255 ];
    Global Maus MausZeiger;
    Global bool msgExit;
    Global Punkt mausPos;
    Global Model3DList *m3dRegister;
    Global TexturList *texturRegister;
#endif
    Global bool istInitialisiert;
    Global ThreadRegister *thRegister;

#ifdef WIN32
    // Gibt die Koordinaten der Maus auf dem Bildschirm zurck
    __declspec( dllexport ) const Punkt &getMausPos();
    // Gibt zurck, ob eine Taste der Maus momentan gedrckt wird
    //  taste: Die Taste, die geprft werden soll
    // Beispiel: getMausStand( M_Links ); (Bentigt include <MausEreignis.h>
    __declspec( dllexport ) bool getMausStand( int taste );
    // Gibt zurck, ob eine Taste auf der tastatur momentan gedrckt wird
    //  taste: Die Taste, die berprft werden soll
    // Beispiel: getTastenStand( T_Enter ); (Bentogt include <TastaturEreignis.h>
    __declspec( dllexport ) bool getTastenStand( unsigned char taste );
    // Legt fest, ob eine Taste auf der tastatur momentan gedrckt wird
    // Der Tastenstand wird vom Framework selbst verwaltet und muss nicht mit dieser Funktion gesetzt werden
    //  taste: Die Taste, deren Status gesetzt werden soll
    //  st: Ob die Taste momentan gedrckt wird. (true), wenn ja. (false) sonnst.
    __declspec( dllexport ) void setTastenStand( unsigned char taste, bool st );
    // Gibt das Model3DData Register des Frameworks ohne erhhten reference Counter zurck
    __declspec( dllexport ) Model3DList *zM3DRegister();
    // Gibt das Textur Register des Frameworks ohne erhhten reference Counter zurck
    __declspec( dllexport ) TexturList *zTexturRegister();
#endif
    // Initialisiert das Framework
    // Wird in der (WinMain) des Frameworks automatisch aufgerufen
    __declspec( dllexport ) void initFramework();
    // Gibt den duch (initFramework) benutzten Arbeitsspeicher wieder frei
    // Wird in der (WinMain) des Frameworks automatisch aufgerufen
    __declspec( dllexport ) void releaseFramework();
    // berprft, ob ein bestimmter Zeiger auf ein Gltiges Thread Objekt zeigt
    //  t: Der zeiger, der berprft werden soll
    //  return: 1, falls der Zeiger in Ordnung ist. 0, falls der Zeiger auf kein existentes Thread Objekt zeigt
    __declspec( dllexport ) bool istThreadOk( Thread *t );
    // Gibt das Thread Register des Frameworks zurck
    __declspec( dllexport ) ThreadRegister *getThreadRegister();
}

#endif