#ifndef InitDatei_H
#define InitDatei_H

#include "Array.h"

namespace Framework
{
    class Text; // Text.h

    // Verwaltet Initialisierungsdateien in denen Bestimmte Variablen im Format name=Wert gespeichert sind
    class InitDatei
    {
    private:
        Text *pfad;
        int feldAnzahl;
        RCArray< Text > *name;
        RCArray< Text > *wert;
        int ref;

    public:
        // Konstruktor
        __declspec( dllexport ) InitDatei();
        // Konstruktor
        //  pfad: Der Pfad zur Datei
        __declspec( dllexport ) InitDatei( Text *pfad );
        // Konstruktor
        //  pfad: Der Pfad zur Datei
        __declspec( dllexport ) InitDatei( const char *pfad );
        // Destruktor
        __declspec( dllexport ) ~InitDatei();
        // Setzt den Pfad zur Datei
        //  pfad: Der Pfad zur Datei
        __declspec( dllexport ) void setPfad( Text *pfad );
        // Setzt den Pfad zur Datei
        //  pfad: Der Pfad zur Datei
        __declspec( dllexport ) void setPfad( const char *pfad );
        // Lhdt die Werte aus der Datei
        //  return: 1, falls das Laden erfolgreich war. 0, wenn ein fehler aufgetreten ist.
        __declspec( dllexport ) bool laden();
        // Fgt der Datei einen Wert hinzu
        //  name: Der Name des Wertes
        //  wert: Der Wert, der gespeichert werden soll
        //  return: 1, falls der Wert erfolgreich hinzugefgt wurde
        __declspec( dllexport ) bool addWert( Text *name, Text *wert );
        // Fgt der Datei einen Wert hinzu
        //  name: Der Name des Wertes
        //  wert: Der Wert, der gespeichert werden soll
        //  return: 1, falls der Wert erfolgreich hinzugefgt wurde
        __declspec( dllexport ) bool addWert( const char *name, const char *wert );
        // ndert einen bestimmten Wert
        //  name: Der Name des Wertes
        //  wert: Der Wert, der gespeichert werden soll
        //  return: 1, falls der Wert vorhanden war und erfolgreich gendert werden konnte
        __declspec( dllexport ) bool setWert( Text *name, Text *wert );
        // ndert einen bestimmten Wert
        //  name: Der Name des Wertes
        //  wert: Der Wert, der gespeichert werden soll
        //  return: 1, falls der Wert vorhanden war und erfolgreich gendert werden konnte
        __declspec( dllexport ) bool setWert( const char *name, const char *wert );
        // ndert einen bestimmten Wert
        //  num: Der Index des Wertes der gendert werden soll
        //  wert: Der Wert, der gespeichert werden soll
        //  return: 1, falls der Wert vorhanden war und erfolgreich gendert werden konnte
        __declspec( dllexport ) bool setWert( int num, Text *wert );
        // ndert einen bestimmten Wert
        //  num: Der Index des Wertes der gendert werden soll
        //  wert: Der Wert, der gespeichert werden soll
        //  return: 1, falls der Wert vorhanden war und erfolgreich gendert werden konnte
        __declspec( dllexport ) bool setWert( int num, const char *wert );
        // Lscht einen bestimmten Wert
        //  name: Der Name des Wertes, der gelscht werden soll
        //  return: 1, falls der Wert vorhanden war und erfolgreich gelscht werden konnte
        __declspec( dllexport ) bool lscheWert( Text *name );
        // Lscht einen bestimmten Wert
        //  name: Der Name des Wertes, der gelscht werden soll
        //  return: 1, falls der Wert vorhanden war und erfolgreich gelscht werden konnte
        __declspec( dllexport ) bool lscheWert( const char *name );
        // Lscht einen bestimmten Wert
        //  num: Der Index des Wertes der gelscht werden soll
        //  return: 1, falls der Wert vorhanden war und erfolgreich gelscht werden konnte
        __declspec( dllexport ) bool lscheWert( int num );
        // Lscht alle Werte aus der Datei
        __declspec( dllexport ) void lscheAlle();
        // Speichert alle Werte in der Datei
        //  return: 1, falls das Speichern erfolgreich war
        __declspec( dllexport ) bool speichern();
        // Gibt die Anzahl der gespeicherten Werte zurck
        __declspec( dllexport ) int getWertAnzahl() const;
        // berprft, ob ein bestimmter Wert existiert
        //  name: Der Name, nach dem gesucht werden soll
        //  return: 1, falls der Wert gefunden wurde
        __declspec( dllexport ) bool wertExistiert( Text *name );
        // berprft, ob ein bestimmter Wert existiert
        //  name: Der Name, nach dem gesucht werden soll
        //  return: 1, falls der Wert gefunden wurde
        __declspec( dllexport ) bool wertExistiert( const char *name );
        // Gibt den Index eines Bestimmten Wertes zurck
        //  name: Der Name des Wertes, nach dem gesucht werden soll
        //  return: -1, falls der Wert nicht gefunden wurde. Der Index des Wertes.
        __declspec( dllexport ) int getWertNummer( Text *name );
        // Gibt den Index eines Bestimmten Wertes zurck
        //  name: Der Name des Wertes, nach dem gesucht werden soll
        //  return: -1, falls der Wert nicht gefunden wurde. Der Index des Wertes.
        __declspec( dllexport ) int getWertNummer( const char *name );
        // Gibt einen bestimmten Wert zurck
        //  name: Der Name des Wertes, der zurckgegeben werden soll
        //  return: 0, falls der Wert nicht gefunden wurde.
        __declspec( dllexport ) Text *getWert( Text *name );
        // Gibt einen bestimmten Wert zurck
        //  name: Der Name des Wertes, der zurckgegeben werden soll
        //  return: 0, falls der Wert nicht gefunden wurde.
        __declspec( dllexport ) Text *getWert( const char *name );
        // Gibt einen bestimmten Wert zurck
        //  num: Der Index des Wertes, der zurckgegeben werden soll
        //  return: 0, falls der Wert nicht gefunden wurde.
        __declspec( dllexport ) Text *getWert( int num );
        // Gibt einen bestimmten Wert zurck
        //  name: Der Name des Wertes, der zurckgegeben werden soll
        //  return: 0, falls der Wert nicht gefunden wurde. Der Wert ohne erhhten Reference Counter
        __declspec( dllexport ) Text *zWert( Text *name );
        // Gibt einen bestimmten Wert zurck
        //  name: Der Name des Wertes, der zurckgegeben werden soll
        //  return: 0, falls der Wert nicht gefunden wurde. Der Wert ohne erhhten Reference Counter
        __declspec( dllexport ) Text *zWert( const char *name );
        // Gibt einen bestimmten Wert zurck
        //  num: Der Index des Wertes, der zurckgegeben werden soll
        //  return: 0, falls der Wert nicht gefunden wurde. Der Wert ohne erhhten Reference Counter
        __declspec( dllexport ) Text *zWert( int num );
        // Gibt den Namen eines bestimmten Wertes zurck
        //  num: Der Index des Wertes, dessen Namen zurckgegeben werden soll
        //  return: 0, falls der Wert nicht gefunden wurde.
        __declspec( dllexport ) Text *getName( int num );
        // Gibt den Namen eines bestimmten Wertes zurck
        //  num: Der Index des Wertes, dessen Namen zurckgegeben werden soll
        //  return: 0, falls der Wert nicht gefunden wurde. Der Name ohne erhhten Reference Counter
        __declspec( dllexport ) Text *zName( int num );
        // Gibt den Pfad zur Datei zurck
        __declspec( dllexport ) Text *getPfad() const;
        // Gibt den Pfad zur Datei ohne erhhten Reference Counter zurck
        __declspec( dllexport ) Text *zPfad() const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) InitDatei *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) InitDatei *release();
    };
}

#endif