#pragma once

#include "Array.h"

namespace Framework
{
    class Textur; // Textur.h

    // Verwaltet alle geladenen Texturdaten, so dass mehrere Zeichnungen die selben Daten benutzen knnen
    class TexturList
    {
    private:
        static int id;
        static CRITICAL_SECTION cs;
        RCArray< Textur > *textures;
        RCArray< Text > *names;
        int ref;

    public:
        // Konstruktor
        TexturList();
        // Destruktor
        ~TexturList();
        // Fgt der Liste eine Textur hinzu
        //  t: Die Textur
        //  name: Der name, unter dem die Textur in der Liste gespeichert wird
        __declspec( dllexport ) bool addTextur( Textur *t, const char *name );
        // Entfernt eine Textur aus der Liste
        //  name: Der Name der Textur
        __declspec( dllexport ) void lscheTextur( const char *name );
        // berprft, ob unter einem bestimmten Namen eine Textur abgespeichert wurde
        //  name: Der Name
        //  return: true, wenn eine Textur mit dem Namen existiert
        __declspec( dllexport ) bool hatTextur( const char *name ) const;
        // Gibt eine bestimmte Textur zurck
        //  name: Der Name der Textur
        __declspec( dllexport ) Textur *getTextur( const char *name ) const;
        // Gibt eine bestimmte Textur zurck
        //  id: Die Id der Textur
        __declspec( dllexport ) Textur *getTextur( int id ) const;
        // Gibt eine bestimmte Textur ohne erhhten Reference Counter zurck
        //  name: Der Name der Textur
        __declspec( dllexport ) Textur *zTextur( const char *name ) const;
        // Gibt eine bestimmte Textur ohne erhhten Reference Counter zurck
        //  id: Die Id der Textur
        __declspec( dllexport ) Textur *zTextur( int id ) const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) TexturList *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) TexturList *release();
        // Initialisiert statische private member. Wird vom Framework automatisch aufgerufen.
        static void init();
        // Lscht statische private member. Wird vom Framework automatisch aufgerufen.
        static void destroy();
    };
}